@@ -18,7 +18,6 @@ import (
 	"context"
 	"net/url"
 	"reflect"
-	"strings"
 	"time"
 
 	"github.com/coreos/prometheus-operator/pkg/alertmanager"
@@ -35,8 +34,10 @@ import (
 	openshiftrouteclientset "github.com/openshift/client-go/route/clientset/versioned"
 	openshiftsecurityclientset "github.com/openshift/client-go/security/clientset/versioned"
 	"github.com/pkg/errors"
+	admissionv1 "k8s.io/api/admissionregistration/v1"
 	appsv1 "k8s.io/api/apps/v1"
 	v1 "k8s.io/api/core/v1"
+	v1betaextensions "k8s.io/api/extensions/v1beta1"
 	rbacv1 "k8s.io/api/rbac/v1"
 	extensionsobj "k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1"
 	apiextensionsclient "k8s.io/apiextensions-apiserver/pkg/client/clientset/clientset"
@@ -48,29 +49,29 @@ import (
 	"k8s.io/client-go/rest"
 	"k8s.io/client-go/tools/cache"
 	"k8s.io/klog"
-	apiregistrationv1beta1 "k8s.io/kube-aggregator/pkg/apis/apiregistration/v1beta1"
+	apiregistrationv1 "k8s.io/kube-aggregator/pkg/apis/apiregistration/v1"
 	aggregatorclient "k8s.io/kube-aggregator/pkg/client/clientset_generated/clientset"
 )
 
 const (
 	deploymentCreateTimeout = 5 * time.Minute
-	metadataPrefix          = "monitoring.openshift.io/"
 )
 
 type Client struct {
-	version           string
-	namespace         string
-	namespaceSelector string
-	kclient           kubernetes.Interface
-	oscclient         openshiftconfigclientset.Interface
-	ossclient         openshiftsecurityclientset.Interface
-	osrclient         openshiftrouteclientset.Interface
-	mclient           monitoring.Interface
-	eclient           apiextensionsclient.Interface
-	aggclient         aggregatorclient.Interface
+	version               string
+	namespace             string
+	userWorkloadNamespace string
+	namespaceSelector     string
+	kclient               kubernetes.Interface
+	oscclient             openshiftconfigclientset.Interface
+	ossclient             openshiftsecurityclientset.Interface
+	osrclient             openshiftrouteclientset.Interface
+	mclient               monitoring.Interface
+	eclient               apiextensionsclient.Interface
+	aggclient             aggregatorclient.Interface
 }
 
-func New(cfg *rest.Config, version string, namespace string, namespaceSelector string) (*Client, error) {
+func New(cfg *rest.Config, version string, namespace, userWorkloadNamespace string, namespaceSelector string) (*Client, error) {
 	mclient, err := monitoring.NewForConfig(cfg)
 	if err != nil {
 		return nil, err
@@ -112,16 +113,17 @@ func New(cfg *rest.Config, version string, namespace string, namespaceSelector s
 	}
 
 	return &Client{
-		version:           version,
-		namespace:         namespace,
-		namespaceSelector: namespaceSelector,
-		kclient:           kclient,
-		oscclient:         oscclient,
-		ossclient:         ossclient,
-		osrclient:         osrclient,
-		mclient:           mclient,
-		eclient:           eclient,
-		aggclient:         aggclient,
+		version:               version,
+		namespace:             namespace,
+		userWorkloadNamespace: userWorkloadNamespace,
+		namespaceSelector:     namespaceSelector,
+		kclient:               kclient,
+		oscclient:             oscclient,
+		ossclient:             ossclient,
+		osrclient:             osrclient,
+		mclient:               mclient,
+		eclient:               eclient,
+		aggclient:             aggclient,
 	}, nil
 }
 
@@ -133,11 +135,6 @@ func (c *Client) Namespace() string {
 	return c.namespace
 }
 
-// ConfigMapListWatch returns a new ListWatch on the ConfigMap resource.
-func (c *Client) ConfigMapListWatch() *cache.ListWatch {
-	return c.ConfigMapListWatchForNamespace(c.namespace)
-}
-
 func (c *Client) ConfigMapListWatchForNamespace(ns string) *cache.ListWatch {
 	return cache.NewListWatchFromClient(c.kclient.CoreV1().RESTClient(), "configmaps", ns, fields.Everything())
 }
@@ -167,6 +164,23 @@ func (c *Client) AssurePrometheusOperatorCRsExist() error {
 	})
 }
 
+func (c *Client) CreateOrUpdateValidatingWebhookConfiguration(w *admissionv1.ValidatingWebhookConfiguration) error {
+	admclient := c.kclient.AdmissionregistrationV1().ValidatingWebhookConfigurations()
+	existing, err := admclient.Get(context.TODO(), w.GetName(), metav1.GetOptions{})
+	if apierrors.IsNotFound(err) {
+		_, err := admclient.Create(context.TODO(), w, metav1.CreateOptions{})
+		return errors.Wrap(err, "creating ValidatingWebhookConfiguration object failed")
+	}
+	if err != nil {
+		return errors.Wrap(err, "retrieving ValidatingWebhookConfiguration object failed")
+	}
+
+	required := w.DeepCopy()
+	required.ResourceVersion = existing.ResourceVersion
+	_, err = admclient.Update(context.TODO(), required, metav1.UpdateOptions{})
+	return errors.Wrap(err, "updating ValidatingWebhookConfiguration object failed")
+}
+
 func (c *Client) CreateOrUpdateSecurityContextConstraints(s *secv1.SecurityContextConstraints) error {
 	sccclient := c.ossclient.SecurityV1().SecurityContextConstraints()
 	existing, err := sccclient.Get(context.TODO(), s.GetName(), metav1.GetOptions{})
@@ -180,7 +194,6 @@ func (c *Client) CreateOrUpdateSecurityContextConstraints(s *secv1.SecurityConte
 
 	// the CRD version of SCC appears to require this.  We can try to chase why later.
 	required := s.DeepCopy()
-	mergeMetadata(&required.ObjectMeta, existing.ObjectMeta)
 	required.ResourceVersion = existing.ResourceVersion
 
 	_, err = sccclient.Update(context.TODO(), required, metav1.UpdateOptions{})
@@ -254,7 +267,7 @@ func (c *Client) NamespacesToMonitor() ([]string, error) {
 
 func (c *Client) CreateOrUpdatePrometheus(p *monv1.Prometheus) error {
 	pclient := c.mclient.MonitoringV1().Prometheuses(p.GetNamespace())
-	existing, err := pclient.Get(context.TODO(), p.GetName(), metav1.GetOptions{})
+	oldProm, err := pclient.Get(context.TODO(), p.GetName(), metav1.GetOptions{})
 	if apierrors.IsNotFound(err) {
 		_, err := pclient.Create(context.TODO(), p, metav1.CreateOptions{})
 		return errors.Wrap(err, "creating Prometheus object failed")
@@ -263,17 +276,14 @@ func (c *Client) CreateOrUpdatePrometheus(p *monv1.Prometheus) error {
 		return errors.Wrap(err, "retrieving Prometheus object failed")
 	}
 
-	required := p.DeepCopy()
-	mergeMetadata(&required.ObjectMeta, existing.ObjectMeta)
-
-	required.ResourceVersion = existing.ResourceVersion
-	_, err = pclient.Update(context.TODO(), required, metav1.UpdateOptions{})
+	p.ResourceVersion = oldProm.ResourceVersion
+	_, err = pclient.Update(context.TODO(), p, metav1.UpdateOptions{})
 	return errors.Wrap(err, "updating Prometheus object failed")
 }
 
 func (c *Client) CreateOrUpdatePrometheusRule(p *monv1.PrometheusRule) error {
 	pclient := c.mclient.MonitoringV1().PrometheusRules(p.GetNamespace())
-	existing, err := pclient.Get(context.TODO(), p.GetName(), metav1.GetOptions{})
+	oldRule, err := pclient.Get(context.TODO(), p.GetName(), metav1.GetOptions{})
 	if apierrors.IsNotFound(err) {
 		_, err := pclient.Create(context.TODO(), p, metav1.CreateOptions{})
 		return errors.Wrap(err, "creating PrometheusRule object failed")
@@ -282,18 +292,15 @@ func (c *Client) CreateOrUpdatePrometheusRule(p *monv1.PrometheusRule) error {
 		return errors.Wrap(err, "retrieving PrometheusRule object failed")
 	}
 
-	required := p.DeepCopy()
-	mergeMetadata(&required.ObjectMeta, existing.ObjectMeta)
+	p.ResourceVersion = oldRule.ResourceVersion
 
-	required.ResourceVersion = existing.ResourceVersion
-
-	_, err = pclient.Update(context.TODO(), required, metav1.UpdateOptions{})
+	_, err = pclient.Update(context.TODO(), p, metav1.UpdateOptions{})
 	return errors.Wrap(err, "updating PrometheusRule object failed")
 }
 
 func (c *Client) CreateOrUpdateAlertmanager(a *monv1.Alertmanager) error {
 	aclient := c.mclient.MonitoringV1().Alertmanagers(a.GetNamespace())
-	existing, err := aclient.Get(context.TODO(), a.GetName(), metav1.GetOptions{})
+	oldAm, err := aclient.Get(context.TODO(), a.GetName(), metav1.GetOptions{})
 	if apierrors.IsNotFound(err) {
 		_, err := aclient.Create(context.TODO(), a, metav1.CreateOptions{})
 		return errors.Wrap(err, "creating Alertmanager object failed")
@@ -302,18 +309,15 @@ func (c *Client) CreateOrUpdateAlertmanager(a *monv1.Alertmanager) error {
 		return errors.Wrap(err, "retrieving Alertmanager object failed")
 	}
 
-	required := a.DeepCopy()
-	mergeMetadata(&required.ObjectMeta, existing.ObjectMeta)
+	a.ResourceVersion = oldAm.ResourceVersion
 
-	required.ResourceVersion = existing.ResourceVersion
-
-	_, err = aclient.Update(context.TODO(), required, metav1.UpdateOptions{})
+	_, err = aclient.Update(context.TODO(), a, metav1.UpdateOptions{})
 	return errors.Wrap(err, "updating Alertmanager object failed")
 }
 
 func (c *Client) CreateOrUpdateThanosRuler(t *monv1.ThanosRuler) error {
 	trclient := c.mclient.MonitoringV1().ThanosRulers(t.GetNamespace())
-	existing, err := trclient.Get(context.TODO(), t.GetName(), metav1.GetOptions{})
+	oldTr, err := trclient.Get(context.TODO(), t.GetName(), metav1.GetOptions{})
 	if apierrors.IsNotFound(err) {
 		_, err := trclient.Create(context.TODO(), t, metav1.CreateOptions{})
 		return errors.Wrap(err, "creating Thanos Ruler object failed")
@@ -322,11 +326,9 @@ func (c *Client) CreateOrUpdateThanosRuler(t *monv1.ThanosRuler) error {
 		return errors.Wrap(err, "retrieving Thanos Ruler object failed")
 	}
 
-	required := t.DeepCopy()
-	mergeMetadata(&required.ObjectMeta, existing.ObjectMeta)
-	required.ResourceVersion = existing.ResourceVersion
+	t.ResourceVersion = oldTr.ResourceVersion
 
-	_, err = trclient.Update(context.TODO(), required, metav1.UpdateOptions{})
+	_, err = trclient.Update(context.TODO(), t, metav1.UpdateOptions{})
 	return errors.Wrap(err, "updating Thanos Ruler object failed")
 }
 
@@ -381,6 +383,15 @@ func (c *Client) DeleteHashedSecret(namespace, prefix, newHash string) error {
 	return nil
 }
 
+func (c *Client) DeleteValidatingWebhook(w *admissionv1.ValidatingWebhookConfiguration) error {
+	err := c.kclient.AdmissionregistrationV1().ValidatingWebhookConfigurations().Delete(context.TODO(), w.GetName(), metav1.DeleteOptions{})
+	if apierrors.IsNotFound(err) {
+		return nil
+	}
+
+	return err
+}
+
 func (c *Client) DeleteDeployment(d *appsv1.Deployment) error {
 	p := metav1.DeletePropagationForeground
 	err := c.kclient.AppsV1().Deployments(d.GetNamespace()).Delete(context.TODO(), d.GetName(), metav1.DeleteOptions{PropagationPolicy: &p})
@@ -538,14 +549,6 @@ func (c *Client) DeleteSecret(s *v1.Secret) error {
 	return err
 }
 
-func (c *Client) DeleteValidatingWebhookConfiguration(name string) error {
-	err := c.kclient.AdmissionregistrationV1().ValidatingWebhookConfigurations().Delete(context.TODO(), name, metav1.DeleteOptions{})
-	if apierrors.IsNotFound(err) {
-		return nil
-	}
-	return err
-}
-
 func (c *Client) WaitForPrometheus(p *monv1.Prometheus) error {
 	var lastErr error
 	if err := wait.Poll(time.Second*10, time.Minute*5, func() (bool, error) {
@@ -646,7 +649,7 @@ func (c *Client) WaitForThanosRuler(t *monv1.ThanosRuler) error {
 }
 
 func (c *Client) CreateOrUpdateDeployment(dep *appsv1.Deployment) error {
-	existing, err := c.kclient.AppsV1().Deployments(dep.GetNamespace()).Get(context.TODO(), dep.GetName(), metav1.GetOptions{})
+	d, err := c.kclient.AppsV1().Deployments(dep.GetNamespace()).Get(context.TODO(), dep.GetName(), metav1.GetOptions{})
 
 	if apierrors.IsNotFound(err) {
 		err = c.CreateDeployment(dep)
@@ -655,24 +658,21 @@ func (c *Client) CreateOrUpdateDeployment(dep *appsv1.Deployment) error {
 	if err != nil {
 		return errors.Wrap(err, "retrieving Deployment object failed")
 	}
-	if reflect.DeepEqual(dep.Spec, existing.Spec) {
-		// Nothing to do, as the currently existing deployment is equivalent to the one that would be applied.
+	if reflect.DeepEqual(dep.Spec, d.Spec) {
+		// Nothing to do, as the currently existing Telemeter client
+		// deployment is equivalent to the one that would be applied.
 		return nil
 	}
-
-	required := dep.DeepCopy()
-	mergeMetadata(&required.ObjectMeta, existing.ObjectMeta)
-
-	err = c.UpdateDeployment(required)
+	err = c.UpdateDeployment(dep)
 	if err != nil {
 		uErr, ok := err.(*apierrors.StatusError)
 		if ok && uErr.ErrStatus.Code == 422 && uErr.ErrStatus.Reason == metav1.StatusReasonInvalid {
 			// try to delete Deployment
-			err = c.DeleteDeployment(existing)
+			err = c.DeleteDeployment(dep)
 			if err != nil {
 				return errors.Wrap(err, "deleting Deployment object failed")
 			}
-			err = c.CreateDeployment(required)
+			err = c.CreateDeployment(dep)
 			if err != nil {
 				return errors.Wrap(err, "creating Deployment object failed after update failed")
 			}
@@ -828,7 +828,7 @@ func (c *Client) WaitForRouteReady(r *routev1.Route) (string, error) {
 }
 
 func (c *Client) CreateOrUpdateDaemonSet(ds *appsv1.DaemonSet) error {
-	existing, err := c.kclient.AppsV1().DaemonSets(ds.GetNamespace()).Get(context.TODO(), ds.GetName(), metav1.GetOptions{})
+	_, err := c.kclient.AppsV1().DaemonSets(ds.GetNamespace()).Get(context.TODO(), ds.GetName(), metav1.GetOptions{})
 	if apierrors.IsNotFound(err) {
 		err = c.CreateDaemonSet(ds)
 		return errors.Wrap(err, "creating DaemonSet object failed")
@@ -837,19 +837,16 @@ func (c *Client) CreateOrUpdateDaemonSet(ds *appsv1.DaemonSet) error {
 		return errors.Wrap(err, "retrieving DaemonSet object failed")
 	}
 
-	required := ds.DeepCopy()
-	mergeMetadata(&required.ObjectMeta, existing.ObjectMeta)
-
-	err = c.UpdateDaemonSet(required)
+	err = c.UpdateDaemonSet(ds)
 	if err != nil {
 		uErr, ok := err.(*apierrors.StatusError)
 		if ok && uErr.ErrStatus.Code == 422 && uErr.ErrStatus.Reason == metav1.StatusReasonInvalid {
 			// try to delete DaemonSet
-			err = c.DeleteDaemonSet(existing)
+			err = c.DeleteDaemonSet(ds)
 			if err != nil {
 				return errors.Wrap(err, "deleting DaemonSet object failed")
 			}
-			err = c.CreateDaemonSet(required)
+			err = c.CreateDaemonSet(ds)
 			if err != nil {
 				return errors.Wrap(err, "creating DaemonSet object failed after update failed")
 			}
@@ -911,7 +908,7 @@ func (c *Client) WaitForDaemonSetRollout(ds *appsv1.DaemonSet) error {
 
 func (c *Client) CreateOrUpdateSecret(s *v1.Secret) error {
 	sClient := c.kclient.CoreV1().Secrets(s.GetNamespace())
-	existing, err := sClient.Get(context.TODO(), s.GetName(), metav1.GetOptions{})
+	_, err := sClient.Get(context.TODO(), s.GetName(), metav1.GetOptions{})
 	if apierrors.IsNotFound(err) {
 		_, err := sClient.Create(context.TODO(), s, metav1.CreateOptions{})
 		return errors.Wrap(err, "creating Secret object failed")
@@ -920,10 +917,7 @@ func (c *Client) CreateOrUpdateSecret(s *v1.Secret) error {
 		return errors.Wrap(err, "retrieving Secret object failed")
 	}
 
-	required := s.DeepCopy()
-	mergeMetadata(&required.ObjectMeta, existing.ObjectMeta)
-
-	_, err = sClient.Update(context.TODO(), required, metav1.UpdateOptions{})
+	_, err = sClient.Update(context.TODO(), s, metav1.UpdateOptions{})
 	return errors.Wrap(err, "updating Secret object failed")
 }
 
@@ -950,7 +944,7 @@ func (c *Client) CreateOrUpdateConfigMapList(cml *v1.ConfigMapList) error {
 
 func (c *Client) CreateOrUpdateConfigMap(cm *v1.ConfigMap) error {
 	cmClient := c.kclient.CoreV1().ConfigMaps(cm.GetNamespace())
-	existing, err := cmClient.Get(context.TODO(), cm.GetName(), metav1.GetOptions{})
+	_, err := cmClient.Get(context.TODO(), cm.GetName(), metav1.GetOptions{})
 	if apierrors.IsNotFound(err) {
 		_, err := cmClient.Create(context.TODO(), cm, metav1.CreateOptions{})
 		return errors.Wrap(err, "creating ConfigMap object failed")
@@ -959,10 +953,22 @@ func (c *Client) CreateOrUpdateConfigMap(cm *v1.ConfigMap) error {
 		return errors.Wrap(err, "retrieving ConfigMap object failed")
 	}
 
-	required := cm.DeepCopy()
-	mergeMetadata(&required.ObjectMeta, existing.ObjectMeta)
+	_, err = cmClient.Update(context.TODO(), cm, metav1.UpdateOptions{})
+	return errors.Wrap(err, "updating ConfigMap object failed")
+}
 
-	_, err = cmClient.Update(context.TODO(), required, metav1.UpdateOptions{})
+func (c *Client) CreateOrUpdateNamespace(n *v1.Namespace) error {
+	nClient := c.kclient.CoreV1().Namespaces()
+	_, err := nClient.Get(context.TODO(), n.GetName(), metav1.GetOptions{})
+	if apierrors.IsNotFound(err) {
+		_, err := nClient.Create(context.TODO(), n, metav1.CreateOptions{})
+		return errors.Wrap(err, "creating Namespace object failed")
+	}
+	if err != nil {
+		return errors.Wrap(err, "retrieving Namespace object failed")
+	}
+
+	_, err = nClient.Update(context.TODO(), n, metav1.UpdateOptions{})
 	return errors.Wrap(err, "updating ConfigMap object failed")
 }
 
@@ -999,7 +1005,7 @@ func (c *Client) CreateIfNotExistConfigMap(cm *v1.ConfigMap) (*v1.ConfigMap, err
 
 func (c *Client) CreateOrUpdateService(svc *v1.Service) error {
 	sclient := c.kclient.CoreV1().Services(svc.GetNamespace())
-	existing, err := sclient.Get(context.TODO(), svc.GetName(), metav1.GetOptions{})
+	s, err := sclient.Get(context.TODO(), svc.GetName(), metav1.GetOptions{})
 	if apierrors.IsNotFound(err) {
 		_, err = sclient.Create(context.TODO(), svc, metav1.CreateOptions{})
 		return errors.Wrap(err, "creating Service object failed")
@@ -1008,22 +1014,35 @@ func (c *Client) CreateOrUpdateService(svc *v1.Service) error {
 		return errors.Wrap(err, "retrieving Service object failed")
 	}
 
-	required := svc.DeepCopy()
-	required.ResourceVersion = existing.ResourceVersion
-	if required.Spec.Type == v1.ServiceTypeClusterIP {
-		required.Spec.ClusterIP = existing.Spec.ClusterIP
+	svc.ResourceVersion = s.ResourceVersion
+	if svc.Spec.Type == v1.ServiceTypeClusterIP {
+		svc.Spec.ClusterIP = s.Spec.ClusterIP
 	}
 
-	if reflect.DeepEqual(required.Spec, existing.Spec) {
+	if reflect.DeepEqual(svc.Spec, s.Spec) && reflect.DeepEqual(svc.Annotations, s.Annotations) && reflect.DeepEqual(svc.Labels, s.Labels) {
 		return nil
 	}
 
-	mergeMetadata(&required.ObjectMeta, existing.ObjectMeta)
-
-	_, err = sclient.Update(context.TODO(), required, metav1.UpdateOptions{})
+	_, err = sclient.Update(context.TODO(), svc, metav1.UpdateOptions{})
 	return errors.Wrap(err, "updating Service object failed")
 }
 
+func (c *Client) CreateOrUpdateEndpoints(endpoints *v1.Endpoints) error {
+	eclient := c.kclient.CoreV1().Endpoints(endpoints.GetNamespace())
+	e, err := eclient.Get(context.TODO(), endpoints.GetName(), metav1.GetOptions{})
+	if apierrors.IsNotFound(err) {
+		_, err = eclient.Create(context.TODO(), endpoints, metav1.CreateOptions{})
+		return errors.Wrap(err, "creating Endpoints object failed")
+	}
+	if err != nil {
+		return errors.Wrap(err, "retrieving Endpoints object failed")
+	}
+
+	endpoints.ResourceVersion = e.ResourceVersion
+	_, err = eclient.Update(context.TODO(), endpoints, metav1.UpdateOptions{})
+	return errors.Wrap(err, "updating Endpoints object failed")
+}
+
 func (c *Client) CreateOrUpdateRoleBinding(rb *rbacv1.RoleBinding) error {
 	rbClient := c.kclient.RbacV1().RoleBindings(rb.GetNamespace())
 	existing, err := rbClient.Get(context.TODO(), rb.GetName(), metav1.GetOptions{})
@@ -1035,21 +1054,22 @@ func (c *Client) CreateOrUpdateRoleBinding(rb *rbacv1.RoleBinding) error {
 		return errors.Wrap(err, "retrieving RoleBinding object failed")
 	}
 
+	// Merge existing annotations with ones defined by operator. Old annotations are preserved.
+	mergo.Merge(&rb.Annotations, existing.Annotations)
+
 	if reflect.DeepEqual(rb.RoleRef, existing.RoleRef) &&
-		reflect.DeepEqual(rb.Subjects, existing.Subjects) {
+		reflect.DeepEqual(rb.Subjects, existing.Subjects) &&
+		reflect.DeepEqual(rb.Labels, existing.Labels) {
 		return nil
 	}
 
-	required := rb.DeepCopy()
-	mergeMetadata(&required.ObjectMeta, existing.ObjectMeta)
-
-	_, err = rbClient.Update(context.TODO(), required, metav1.UpdateOptions{})
+	_, err = rbClient.Update(context.TODO(), rb, metav1.UpdateOptions{})
 	return errors.Wrap(err, "updating RoleBinding object failed")
 }
 
 func (c *Client) CreateOrUpdateRole(r *rbacv1.Role) error {
 	rClient := c.kclient.RbacV1().Roles(r.GetNamespace())
-	existing, err := rClient.Get(context.TODO(), r.GetName(), metav1.GetOptions{})
+	_, err := rClient.Get(context.TODO(), r.GetName(), metav1.GetOptions{})
 	if apierrors.IsNotFound(err) {
 		_, err := rClient.Create(context.TODO(), r, metav1.CreateOptions{})
 		return errors.Wrap(err, "creating Role object failed")
@@ -1058,16 +1078,13 @@ func (c *Client) CreateOrUpdateRole(r *rbacv1.Role) error {
 		return errors.Wrap(err, "retrieving Role object failed")
 	}
 
-	required := r.DeepCopy()
-	mergeMetadata(&required.ObjectMeta, existing.ObjectMeta)
-
-	_, err = rClient.Update(context.TODO(), required, metav1.UpdateOptions{})
+	_, err = rClient.Update(context.TODO(), r, metav1.UpdateOptions{})
 	return errors.Wrap(err, "updating Role object failed")
 }
 
 func (c *Client) CreateOrUpdateClusterRole(cr *rbacv1.ClusterRole) error {
 	crClient := c.kclient.RbacV1().ClusterRoles()
-	existing, err := crClient.Get(context.TODO(), cr.GetName(), metav1.GetOptions{})
+	_, err := crClient.Get(context.TODO(), cr.GetName(), metav1.GetOptions{})
 	if apierrors.IsNotFound(err) {
 		_, err := crClient.Create(context.TODO(), cr, metav1.CreateOptions{})
 		return errors.Wrap(err, "creating ClusterRole object failed")
@@ -1076,10 +1093,7 @@ func (c *Client) CreateOrUpdateClusterRole(cr *rbacv1.ClusterRole) error {
 		return errors.Wrap(err, "retrieving ClusterRole object failed")
 	}
 
-	required := cr.DeepCopy()
-	mergeMetadata(&required.ObjectMeta, existing.ObjectMeta)
-
-	_, err = crClient.Update(context.TODO(), required, metav1.UpdateOptions{})
+	_, err = crClient.Update(context.TODO(), cr, metav1.UpdateOptions{})
 	return errors.Wrap(err, "updating ClusterRole object failed")
 }
 
@@ -1095,19 +1109,20 @@ func (c *Client) CreateOrUpdateClusterRoleBinding(crb *rbacv1.ClusterRoleBinding
 	}
 
 	if reflect.DeepEqual(crb.RoleRef, existing.RoleRef) &&
-		reflect.DeepEqual(crb.Subjects, existing.Subjects) {
+		reflect.DeepEqual(crb.Subjects, existing.Subjects) &&
+		reflect.DeepEqual(crb.Labels, existing.Labels) {
 		return nil
 	}
 
-	required := crb.DeepCopy()
-	mergeMetadata(&required.ObjectMeta, existing.ObjectMeta)
+	// Merge existing annotations with ones defined by operator. Old annotations are preserved.
+	mergo.Merge(&crb.Annotations, existing.Annotations)
 
 	err = crbClient.Delete(context.TODO(), crb.Name, metav1.DeleteOptions{})
 	if err != nil {
 		return errors.Wrap(err, "deleting ClusterRoleBinding object failed")
 	}
 
-	_, err = crbClient.Create(context.TODO(), required, metav1.CreateOptions{})
+	_, err = crbClient.Create(context.TODO(), crb, metav1.CreateOptions{})
 	return errors.Wrap(err, "updating ClusterRoleBinding object failed")
 }
 
@@ -1140,7 +1155,7 @@ func (c *Client) CreateOrUpdateServiceAccount(sa *v1.ServiceAccount) error {
 
 func (c *Client) CreateOrUpdateServiceMonitor(sm *monv1.ServiceMonitor) error {
 	smClient := c.mclient.MonitoringV1().ServiceMonitors(sm.GetNamespace())
-	existing, err := smClient.Get(context.TODO(), sm.GetName(), metav1.GetOptions{})
+	oldSm, err := smClient.Get(context.TODO(), sm.GetName(), metav1.GetOptions{})
 	if apierrors.IsNotFound(err) {
 		_, err := smClient.Create(context.TODO(), sm, metav1.CreateOptions{})
 		return errors.Wrap(err, "creating ServiceMonitor object failed")
@@ -1149,17 +1164,29 @@ func (c *Client) CreateOrUpdateServiceMonitor(sm *monv1.ServiceMonitor) error {
 		return errors.Wrap(err, "retrieving ServiceMonitor object failed")
 	}
 
-	required := sm.DeepCopy()
-	mergeMetadata(&required.ObjectMeta, existing.ObjectMeta)
-
-	required.ResourceVersion = existing.ResourceVersion
-	_, err = smClient.Update(context.TODO(), required, metav1.UpdateOptions{})
+	sm.ResourceVersion = oldSm.ResourceVersion
+	_, err = smClient.Update(context.TODO(), sm, metav1.UpdateOptions{})
 	return errors.Wrap(err, "updating ServiceMonitor object failed")
 }
 
-func (c *Client) CreateOrUpdateAPIService(apiService *apiregistrationv1beta1.APIService) error {
-	apsc := c.aggclient.ApiregistrationV1beta1().APIServices()
-	existing, err := apsc.Get(context.TODO(), apiService.GetName(), metav1.GetOptions{})
+func (c *Client) CreateOrUpdateIngress(ing *v1betaextensions.Ingress) error {
+	ic := c.kclient.ExtensionsV1beta1().Ingresses(ing.GetNamespace())
+	_, err := ic.Get(context.TODO(), ing.GetName(), metav1.GetOptions{})
+	if apierrors.IsNotFound(err) {
+		_, err = ic.Create(context.TODO(), ing, metav1.CreateOptions{})
+		return errors.Wrap(err, "creating Ingress object failed")
+	}
+	if err != nil {
+		return errors.Wrap(err, "retrieving Ingress object failed")
+	}
+
+	_, err = ic.Update(context.TODO(), ing, metav1.UpdateOptions{})
+	return errors.Wrap(err, "updating Ingress object failed")
+}
+
+func (c *Client) CreateOrUpdateAPIService(apiService *apiregistrationv1.APIService) error {
+	apsc := c.aggclient.ApiregistrationV1().APIServices()
+	oldAPIService, err := apsc.Get(context.TODO(), apiService.GetName(), metav1.GetOptions{})
 	if apierrors.IsNotFound(err) {
 		_, err = apsc.Create(context.TODO(), apiService, metav1.CreateOptions{})
 		return errors.Wrap(err, "creating APIService object failed")
@@ -1168,12 +1195,11 @@ func (c *Client) CreateOrUpdateAPIService(apiService *apiregistrationv1beta1.API
 		return errors.Wrap(err, "retrieving APIService object failed")
 	}
 
-	required := apiService.DeepCopy()
-	required.ResourceVersion = existing.ResourceVersion
-	if len(existing.Spec.CABundle) > 0 {
-		required.Spec.CABundle = existing.Spec.CABundle
+	apiService.ResourceVersion = oldAPIService.ResourceVersion
+	if len(oldAPIService.Spec.CABundle) > 0 {
+		apiService.Spec.CABundle = oldAPIService.Spec.CABundle
 	}
-	_, err = apsc.Update(context.TODO(), required, metav1.UpdateOptions{})
+	_, err = apsc.Update(context.TODO(), apiService, metav1.UpdateOptions{})
 	return errors.Wrap(err, "updating APIService object failed")
 
 }
@@ -1207,7 +1233,7 @@ func (c *Client) CRDReady(crd *extensionsobj.CustomResourceDefinition) (bool, er
 }
 
 func (c *Client) StatusReporter() *StatusReporter {
-	return NewStatusReporter(c.oscclient.ConfigV1().ClusterOperators(), "monitoring", c.namespace, c.version)
+	return NewStatusReporter(c.oscclient.ConfigV1().ClusterOperators(), "monitoring", c.namespace, c.userWorkloadNamespace, c.version)
 }
 
 func (c *Client) DeleteRoleBinding(binding *rbacv1.RoleBinding) error {
@@ -1227,24 +1253,3 @@ func (c *Client) DeleteRole(role *rbacv1.Role) error {
 
 	return err
 }
-
-// mergeMetadata merges labels and annotations from `existing` map into `required` one where `required` has precedence
-// over `existing` keys and values. Additionally function performs filtering of labels and annotations from `exiting` map
-// where keys starting from string defined in `metadataPrefix` are deleted. This prevents issues with preserving stale
-// metadata defined by the operator
-func mergeMetadata(required *metav1.ObjectMeta, existing metav1.ObjectMeta) {
-	for k := range existing.Annotations {
-		if strings.HasPrefix(k, metadataPrefix) {
-			delete(existing.Annotations, k)
-		}
-	}
-
-	for k := range existing.Labels {
-		if strings.HasPrefix(k, metadataPrefix) {
-			delete(existing.Labels, k)
-		}
-	}
-
-	mergo.Merge(&required.Annotations, existing.Annotations)
-	mergo.Merge(&required.Labels, existing.Labels)
-}
